"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const promisified_1 = require("./promisified");
const CHUNK_SIZE = 1048576; // yieldLinesFromFile is reading files in batches of this size
const NEW_LINE = 0x0a;
const CARRIAGE_RETURN = 0x0d;
async function unlink(path) {
    return promisified_1.default.unlink(path);
}
exports.unlink = unlink;
async function mkdir(path) {
    return promisified_1.default.mkdir(path);
}
exports.mkdir = mkdir;
async function exists(path) {
    return promisified_1.default.exists(path);
}
exports.exists = exists;
async function stat(path) {
    return promisified_1.default.stat(path);
}
exports.stat = stat;
async function copy(src, dst, throwIfExists) {
    return promisified_1.default.copy(src, dst, throwIfExists ? fs_1.constants.COPYFILE_EXCL : undefined);
}
exports.copy = copy;
async function symlink(src, linkPath, type) {
    return promisified_1.default.symlink(src, linkPath, type);
}
exports.symlink = symlink;
async function createIfDoesntExist(folder) {
    if (await exists(folder))
        return;
    return mkdir(folder);
}
exports.createIfDoesntExist = createIfDoesntExist;
async function ls(path, absolute = false) {
    const files = await promisified_1.default.readdir(path);
    const pathStr = path.toString();
    return absolute ? files.map(f => path_1.join(pathStr, f)) : files;
}
exports.ls = ls;
/**
 * The path.extname() method returns the extension of the path, from the last occurrence of the . (period) character to end of string in the last portion of the path. If there is no . in the last portion of the path, or if the first character of the basename of path (see path.basename()) is ., then an empty string is returned.
 */
function getFileExtension(path) {
    return path_1.extname(path.toString());
}
exports.getFileExtension = getFileExtension;
function getFileName(path, withoutExtension = false) {
    return path_1.basename(path.toString(), withoutExtension ? getFileExtension(path) : undefined);
}
exports.getFileName = getFileName;
function getFolderName(path) {
    return path_1.dirname(path.toString());
}
exports.getFolderName = getFolderName;
function changeFileExtension(path, newExtension) {
    const ext = getFileExtension(path);
    const end = new RegExp(`\\${ext}$`);
    return path.toString().replace(end, newExtension);
}
exports.changeFileExtension = changeFileExtension;
async function read(path, encoding = "utf8") {
    return (await promisified_1.default.readFile(path, encoding ? { encoding } : {})).toString();
}
exports.read = read;
async function readBuffer(path) {
    // tslint:disable-next-line:no-null-keyword
    return promisified_1.default.readFile(path, { encoding: null });
}
exports.readBuffer = readBuffer;
/**
 * @throws Error
 */
async function readJSON(path) {
    return JSON.parse(await read(path, "utf8"));
}
exports.readJSON = readJSON;
/** whitelist - what files to keep, defaults to /a^/ which matches nothing */
exports.clearFolder = async (folder, whitelist = /a^/) => {
    const files = await ls(folder, true);
    const deletionJobs = files.map(async (f) => whitelist.test(f) ? Promise.resolve() : unlink(f));
    return Promise.all(deletionJobs);
};
async function write(path, 
// tslint:disable-next-line:no-any
data, encoding = "utf8") {
    return promisified_1.default.writeFile(path, data.toString(), encoding ? { encoding } : {});
}
exports.write = write;
function isEOL(value) {
    return value === NEW_LINE || value === CARRIAGE_RETURN;
}
exports.isEOL = isEOL;
function* yieldLinesFromFile(path) {
    const readBuffer = new Buffer(CHUNK_SIZE);
    if (!fs_1.existsSync(path)) {
        throw new Error(`No such file or directory '${path}'`);
    }
    const fd = fs_1.openSync(path, "r");
    let readBytes = 0;
    let remainder = "";
    let n = 0;
    do {
        readBytes = fs_1.readSync(fd, readBuffer, 0, CHUNK_SIZE, CHUNK_SIZE * n++);
        let start = 0;
        for (let i = 0; i < readBytes; i++) {
            if (isEOL(readBuffer[i])) {
                const end = i;
                const line = remainder + readBuffer.slice(start, end).toString();
                remainder = "";
                yield line;
                i += readBuffer[i] === CARRIAGE_RETURN ? 1 : 0;
                start = i + 1;
            }
        }
        if (start !== readBytes) {
            remainder = remainder + readBuffer.slice(start, readBytes).toString();
        }
    } while (readBytes > 0);
    if (remainder !== "") {
        yield remainder;
    }
}
exports.yieldLinesFromFile = yieldLinesFromFile;
//# sourceMappingURL=index.js.map