"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const query_string_1 = require("query-string");
const ajax_1 = require("../ajax");
class RefreshTokenExpiredError extends Error {
    constructor() {
        super("Refresh Token Expired");
    }
}
exports.RefreshTokenExpiredError = RefreshTokenExpiredError;
class Oauth2 {
    constructor(config, lastToken) {
        this.config = config;
        if (lastToken)
            this.lastToken = lastToken;
        const basicAuth = Buffer.from(`${this.config.clientID}:${this.config.clientSecret}`).toString("base64");
        this.basicAuthHeaders = {
            Authorization: `Basic ${basicAuth}`
        };
    }
    async postTokenRequest(form) {
        const result = (await ajax_1.request("post", this.config.tokenURL, form, true, this.basicAuthHeaders)).json();
        return result;
    }
    /**
     * @throws RefreshTokenExpiredError if no refreshToken has expired and no `code` param is given
     */
    async requestToken(code) {
        if (this.fetchingToken)
            return this.fetchingToken;
        if (!code)
            throw new RefreshTokenExpiredError();
        const form = {
            code,
            redirect_uri: this.config.redirectURI,
            grant_type: "authorization_code"
        };
        this.fetchingToken = this.postTokenRequest(form);
        try {
            const tokenAnswer = await this.fetchingToken;
            this.lastToken = tokenAnswer;
            return tokenAnswer;
        }
        catch (e) {
            // Access token expired
            if (this.lastToken) {
                try {
                    this.fetchingToken = this.refreshToken(this.lastToken.refresh_token);
                    return this.fetchingToken;
                }
                catch (e) {
                    // Refresh token expired
                    return this.requestToken(code);
                }
            }
        }
        return this.fetchingToken;
    }
    async requestAuth(state, query) {
        return ajax_1.get(this.getRequestAuthURL(state, query));
    }
    getRequestAuthURL(state, query) {
        const { redirectURI, clientID, authorizeURL } = this.config;
        const qs = query_string_1.stringify({
            client_id: clientID,
            response_type: "code",
            redirect_uri: redirectURI,
            state,
            ...query
        });
        return `${authorizeURL}?${qs}`;
    }
    async refreshToken(refreshToken) {
        const form = {
            grant_type: "refresh_token",
            refresh_token: refreshToken
        };
        return this.postTokenRequest(form);
    }
    async getTokenAuthHeaders() {
        if (this.lastToken) {
            return {
                Authorization: `Bearer ${this.lastToken.access_token}`
            };
        }
        else {
            await this.requestToken();
            return this.getTokenAuthHeaders();
        }
    }
    async mergeHeaders(customHeaders) {
        const authHeaders = await this.getTokenAuthHeaders();
        return {
            ...authHeaders,
            ...customHeaders
        };
    }
    async authenticatedGet(url, customHeaders) {
        return ajax_1.get(url, await this.mergeHeaders(customHeaders));
    }
    async authenticatedPost(url, body, form = false, customHeaders) {
        return ajax_1.post(url, body, form, await this.mergeHeaders(customHeaders));
    }
}
exports.default = Oauth2;
//# sourceMappingURL=index.js.map